/*
Copyright (©) 2003-2014 Teus Benschop.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
  
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


$(document).ready (function () {
  rangy.init ();
  navigationNewPassage ();
  $ ("#editor").on ("paste cut keydown", editorChanged);
  $ (window).on ("unload", editorUnload);
  $ ("#editor").on ("paste", function (e) {
    var data = e.originalEvent.clipboardData.getData ('Text');
    e.preventDefault();
    document.execCommand ("insertHTML", false, data);
  });
  editorIdPollerTimerStart ();
  $ ("#editor").on ("paste cut click", editorCaretChanged);
  $ ("#editor").on ("keydown", editorHandleKeyDown);
  $ ("#editor").on ("focus", editorWindowFocused);
  $ ("#editor").on ("blur", editorWindowBlurred);
  $ ("#editor").focus ();
  editorBindUnselectable ();
  $ ("#stylebutton").on ("click", handleStyleButton);
  $ (window).on ("keydown", editorPageKeyDown);
  $ (window).scroll (function () {
    $ ('#editorinnerheader').toggleClass('editorheaderscroll', $ (window).scrollTop () > $ ('#editorheader').offset ().top);
  });  
});


var editorBible;
var editorBook;
var editorNavigationBook;
var editorChapter;
var editorNavigationChapter;
var editorNavigationVerse;
var editorServerText;
var editorIdChapter = 0;
var editorReload = false;
var editorCaretPosition = 0;
var editorPreviousWidth;
var editorPreviousHeight;
var editorTextChanged = false;


function navigationNewPassage ()
{
  if (typeof navigationBook != 'undefined') {
    editorNavigationBook = navigationBook;
    editorNavigationChapter = navigationChapter;
    editorNavigationVerse = navigationVerse;
  } else if (parent.window.navigationBook != 'undefined') {
    editorNavigationBook = parent.window.navigationBook;
    editorNavigationChapter = parent.window.navigationChapter;
    editorNavigationVerse = parent.window.navigationVerse;
  } else {
    return;
  }
  editorSaveChapter ();
  editorReload = false;
  editorLoadChapter ();
  positionCaretViaAjax ();
}


function editorLoadChapter ()
{
  if ((editorNavigationBook != editorBook) || (editorNavigationChapter != editorChapter) || editorReload) {
    editorBible = navigationBible;
    editorBook = editorNavigationBook;
    editorChapter = editorNavigationChapter;
    editorIdChapter = 0;
    editorCaretPosition = getCaretPosition ();
    // $ ("#editor").attr ("contenteditable", "false"); Trouble on Chrome.
    $.ajax ({
      url: "load.php",
      type: "GET",
      data: { bible: editorBible, book: editorBook, chapter: editorChapter },
      success: function (response) {
        // Only load new text when it is different.
        if (response != editorGetHtml ()) {
          $ ("#editor").empty ();
          $ ("#editor").append (response);
          editorStatus (editorChapterLoaded);
        }
        // $ ("#editor").attr ("contenteditable", "true"); Trouble on Chrome.
        editorServerText = response;
        if (editorReload) {
          positionCaret (editorCaretPosition);
        } else {
          positionCaretViaAjax ();
        }
        editorReload = false;
      },
    });
  }
}


function editorUnload ()
{
  editorSaveChapter (true);
}


function editorSaveChapter (sync)
{
  if (!editorWriteAccess) return;
  editorTextChanged = false;
  if (!editorBible) return;
  if (!editorBook) return;
  var html = editorGetHtml ();
  if (html == editorServerText) return;
  editorStatus (editorChapterSaving);
  editorServerText = html;
  editorIdChapter = 0;
  editorIdPollerTimerStop ();
  var async = true;
  if (sync) async = false;
  $.ajax ({
    url: "save.php",
    type: "POST",
    async: async,
    data: { bible: editorBible, book: editorBook, chapter: editorChapter, html: html },
    success: function (response) {
      editorStatus (response);
    },
    error: function (jqXHR, textStatus, errorThrown) {
      editorStatus (editorChapterRetrying);
      editorServerText = "";
      editorChanged ();
    },
    complete: function (xhr, status) {
      editorIdPollerTimerStart ();
    },
  });
}


function editorChanged ()
{
  if (!editorWriteAccess) return;
  editorTextChanged = true;
  editorChangedTimerStart ();
  editorScrollerTimerStart ();
}


function editorStatus (text)
{
  $ ("#editorstatus").empty ();
  $ ("#editorstatus").append (text);
}


function editorEditorPollId ()
{
  $.ajax ({
    url: "id.php",
    type: "GET",
    data: { bible: editorBible, book: editorBook, chapter: editorChapter },
    success: function (response) {
      if (editorIdChapter != 0) {
        if (response != editorIdChapter) {
          editorReload = true;
          editorLoadChapter ();
          editorIdChapter = 0;
        }
      }
      editorIdChapter = response;
    },
    complete: function (xhr, status) {
      editorIdPollerTimerStart ();
    }
  });
}


function editorCaretChanged (event)
{
  editorCaretTimerStart ();
  editorScrollerTimerStart ();
}


function editorHandleKeyDown (event)
{
  // Ctrl-G: No action.
  if ((event.ctrlKey == true) && (event.keyCode == 71)) {
    return;
  }
  // Alt / Ctrl / Shift: No action.
  if (event.keyCode == 18) return;
  if (event.keyCode == 17) return;
  if (event.keyCode == 16) return;

  // Do nothing for next/previous verse.
  if (event.altKey) {
    if (event.keyCode == 38) return;
    if (event.keyCode == 40) return;
  }
  
  editorCaretChanged ();
}


function editorPageKeyDown (event)
{
  if (!editorWriteAccess) return;
  // Ctrl-S: Style.
  if ((event.ctrlKey == true) && (event.keyCode == 83)) {
    handleStyleButton ();
    return false;
  }
  // Escape.
  if (event.keyCode == 27) {
    editorClearStyles ();
  }
}


function editorHandleCaret ()
{
  if (editorTextChanged) {
    editorCaretChanged ();
    return;
  }
  if ($ ("#editor").is (":focus")) {
    var offset = getCaretPosition ();
    $.ajax ({
      url: "offset.php",
      type: "GET",
      data: { bible: editorBible, book: editorBook, chapter: editorChapter, offset: offset },
      success: function (response) {
      }
    });
  }
  editorActiveStylesFeedback ();
}


function positionCaretViaAjax ()
{
  $.ajax ({
    url: "focus.php",
    type: "GET",
    data: { bible: editorBible, book: editorBook, chapter: editorChapter },
    success: function (response) {
      if (response != "") {
        response = $.parseJSON (response);
        var start = response ["start"];
        var end = response ["end"];
        var offset = getCaretPosition ();
        if ((offset < start) || (offset > end)) {
          positionCaret (start);
        }
      }
      editorScrollerTimerStart ();
    }
  });
}


function getCaretPosition ()
{
  var position = undefined;
  var editor = $ ("#editor");
  if (editor.is (":focus")) {
    var element = editor.get (0);
    position = getCaretCharacterOffsetWithin (element);
  }
  return position;
}


function getCaretCharacterOffsetWithin (element)
{
  var caretOffset = 0;
  if (typeof window.getSelection != "undefined") {
    var range = window.getSelection().getRangeAt(0);
    var preCaretRange = range.cloneRange();
    preCaretRange.selectNodeContents(element);
    preCaretRange.setEnd(range.endContainer, range.endOffset);
    caretOffset = preCaretRange.toString().length;
  } else if (typeof document.selection != "undefined" && document.selection.type != "Control") {
    var textRange = document.selection.createRange();
    var preCaretTextRange = document.body.createTextRange();
    preCaretTextRange.moveToElementText(element);
    preCaretTextRange.setEndPoint("EndToEnd", textRange);
    caretOffset = preCaretTextRange.text.length;
  }
  return caretOffset;
}


function positionCaret (position)
{
  var currentPosition = getCaretPosition ();
  if (currentPosition == undefined) return;
  if (position == undefined) return;
  var selection = rangy.getSelection ();
  selection.move ("character", position - currentPosition);
}


function editorWindowFocused ()
{
  positionCaretViaAjax ();
}


function editorWindowBlurred ()
{
}


function editorScrollCaretIntoView ()
{
  $ (".v").each (function (index) {
    var element = $(this);
    var text = element[0].innerText;
    if (text == editorNavigationVerse) {
      element.attr ("id", "focus");
      var offset = element.offset ();
      var verseTop = offset.top;
      var viewportHeight = $(window).height ();
      var scrollTo = verseTop - (viewportHeight / 2);
      var currentScrollTop = $ ("body").scrollTop ();
      var lowerBoundary = currentScrollTop - (viewportHeight / 10);
      var upperBoundary = currentScrollTop + (viewportHeight / 10);
      if ((scrollTo < lowerBoundary) || (scrollTo > upperBoundary)) {
        $ ("body").animate ({ scrollTop: scrollTo }, 500);
      }
    } else {
      if (element.attr ("id") == "focus") {
        element.removeAttr ("id");
      }
    }
  });
  if (editorNavigationVerse == 0) {
    $ ("body").animate ({ scrollTop: scrollTo }, 0);
  }
}


function handleStyleButton ()
{
  if (!editorWriteAccess) return;
  $.get ("styles.php", function (response) {
    editorShowResponse (response);
    editorBindUnselectable ();
    dynamicClickHandlers ();
  });
  return false;
}


function editorBindUnselectable ()
{
  var elements = $ (".unselectable");
  elements.off ("mousedown");
  elements.on ("mousedown", function (event) {
    event.preventDefault();
  });
}


function editorShowResponse (response)
{
  if (!editorWriteAccess) return;
  $ ("#stylebutton").hide ();
  $ ("#nostyles").hide ();
  var area = $ ("#stylesarea");
  area.empty ();
  area.append (response);
}


function editorClearStyles ()
{
  var area = $ ("#stylesarea");
  area.empty ();
  $ ("#stylebutton").show ();
  $ ("#nostyles").show ();
}


function dynamicClickHandlers ()
{
  var elements = $ ("#stylesarea > a");
  elements.on ("click", function (event) {
    event.preventDefault();
    editorClearStyles ();
    $ ("#editor").focus ();
    var href = event.currentTarget.href;
    href = href.substring (href.lastIndexOf ('/') + 1);
    if (href == "cancel") return;
    if (href == "all") {
      displayAllStyles ();
    } else {
      requestStyle (href);
    }
  });

  $ ("#styleslist").on ("change", function (event) {
    var selection = $ ("#styleslist option:selected").text ();
    var style = selection.substring (0, selection.indexOf (" "));
    event.preventDefault();
    editorClearStyles ();
    $ ("#editor").focus ();
    requestStyle (style);
  });
}


function requestStyle (style)
{
  $.get ("styles.php?style=" + style, function (response) {
    response = $.parseJSON (response);
    var style = response ["style"];
    var action = response ["action"];
    if (action == "p") {
      applyParagraphStyle (style);
      editorChanged ();
    } else if (action == 'c') {
      applyCharacterStyle (style);
      editorChanged ();
    } else if (action == 'n') {
      applyNotesStyle (style);
      editorChanged ();
    } else if (action == "m") {
      applyMonoStyle (style);
      editorChanged ();
    }
  });
}


function displayAllStyles ()
{
  $.get ("styles.php?all=", function (response) {
    editorShowResponse (response);
    editorBindUnselectable ();
    dynamicClickHandlers ();
  });
}


function applyParagraphStyle (style)
{
  if (!editorWriteAccess) return;
  $ ("#editor").focus ();
  var parent = rangy.getSelection().anchorNode.parentElement;
  parent = $ (parent);
  var paragraph = parent.closest ("p");
  paragraph.removeClass ();
  paragraph.addClass (style);
}


function applyCharacterStyle (style)
{
  if (!editorWriteAccess) return;
  $ ("#editor").focus ();
  var cssApplier = rangy.createCssClassApplier (style);
  cssApplier.toggleSelection ();
}


function applyMonoStyle (style)
{
  if (!editorWriteAccess) return;

  $ ("#editor").focus ();

  var parent = rangy.getSelection().anchorNode.parentElement;
  parent = $ (parent);
  var paragraph = parent.closest ("p");

  paragraph.removeClass ();
  paragraph.addClass ("mono");

  var text = paragraph.text ();

  var char = text.substring (0, 1);
  if (char == "\\") {
    text = text.substring (1, 10000);
    var pos = text.indexOf (' ');
    text = text.substring (pos + 1, 10000);
  }
  text = "\\" + style + " " + text;
  paragraph.text (text);
}


function editorGetHtml ()
{
  var html = $ ("#editor").html ();
  html = html.replace (' id="focus"', '');
  return html;
}


// Timer section.


function editorSuspendAllTimers ()
{
  editorIdPollerTimerStop ();
  editorChangedTimerStop ();
  editorCaretTimerStop ();
  editorScrollerTimerStop ();
}


var editorIdPollerTimerId;

function editorIdPollerTimerStop ()
{
  if (editorIdPollerTimerId) clearTimeout (editorIdPollerTimerId);
}

function editorIdPollerTimerStart ()
{
  editorIdPollerTimerStop ();
  editorIdPollerTimerId = setTimeout (editorEditorPollId, 1000);
}


var editorChangedTimerId;

function editorChangedTimerStop ()
{
  if (editorChangedTimerId) clearTimeout (editorChangedTimerId);
}

function editorChangedTimerStart ()
{
  editorChangedTimerStop ();
  editorChangedTimerId = setTimeout (editorSaveChapter, 1000);
}


var editorCaretTimerId;

function editorCaretTimerStop ()
{
  if (editorCaretTimerId) clearTimeout (editorCaretTimerId);
}

function editorCaretTimerStart ()
{
  editorCaretTimerStop ();
  editorCaretTimerId = setTimeout (editorHandleCaret, 500);
}


var editorScrollerTimerId;

function editorScrollerTimerStop ()
{
  if (editorScrollerTimerId) clearTimeout (editorScrollerTimerId);
}


function editorScrollerTimerStart ()
{
  editorScrollerTimerStop ();
  editorScrollerTimerId = setTimeout (editorScrollCaretIntoView, 100);
}


// Notes section.


function applyNotesStyle (style)
{
  if (!editorWriteAccess) return;

  $ ("#editor").focus ();

  // Check for / append notes section.
  var notes = $ ("#notes");
  if (notes.length == 0) {
    $ ("#editor").append ('<div id="notes"><hr></div>');
  }

  // Get available notes identifier.
  var availableId = 0;
  $ ("div > p > a").each (function () {
    var href = $ (this).attr ("href");
    if (href.indexOf ("note") > 0) {
      href = href.match(/\d+$/) [0];
      if (href > availableId) {
        availableId = href;
      }
    }
  });
  availableId++;
  console.log (availableId);

  // Insert note caller at caret.
  var html = '<a href="#note' + availableId + '" id="citation' + availableId + '" class="superscript">' + style + '</a>';
  insertHtmlAtCaret (html);
  
  // Append note text to notes section.
  html = '<p class="' + style + '"><a href="#citation' + availableId + '" id="note' + availableId + '">' + style + '</a><span> </span><span>+ </span><span> </span></p>';
  $ ("#notes").append (html);
}


// Insert html at caret.


function insertHtmlAtCaret (html) 
{
  var sel, range;
  if (window.getSelection) {
    // IE9 and non-IE
    sel = window.getSelection();
    if (sel.getRangeAt && sel.rangeCount) {
      range = sel.getRangeAt(0);
      range.deleteContents();

      // Range.createContextualFragment() would be useful here but is
      // only relatively recently standardized and is not supported in
      // some browsers (IE9, for one)
      var el = document.createElement("div");
      el.innerHTML = html;
      var frag = document.createDocumentFragment(), node, lastNode;
      while ( (node = el.firstChild) ) {
        lastNode = frag.appendChild(node);
      }
      var firstNode = frag.firstChild;
      range.insertNode(frag);
      
      // Preserve the selection
      if (lastNode) {
        range = range.cloneRange();
        range.setStartAfter(lastNode);
        range.collapse(true);
        sel.removeAllRanges();
        sel.addRange(range);
      }
    }
  } else if ((sel = document.selection) && sel.type != "Control") {
    // IE < 9
    var originalRange = sel.createRange();
    originalRange.collapse(true);
    sel.createRange().pasteHTML(html);
  }
}


// Active styles feedback.


function editorActiveStylesFeedback ()
{
  var editor = $ ("#editor");
  if (editor.is (":focus")) {
    var parent = rangy.getSelection().anchorNode.parentElement;
    parent = $ (parent);
    var paragraph = parent.closest ("p");
    var pname = paragraph.attr ('class');
    var span = parent.closest ("span");
    var cname = span.attr ("class");
    if (cname == undefined) cname = "";
    var element = $ ("#activestyles");
    element.text (pname + " " + cname);
  }
}

